// Copyright (c) Microsoft Open Technologies, Inc.  All rights reserved.  
// Licensed under the BSD 2-Clause License.  
// See License.txt in the project root for license information.

#include "common.h"

#include "GpioInterrupt.h"
#include "DmapSupport.h"
#include "Robuffer.h"

/// Method to attach to an interrupt on a GPIO port bit.
HRESULT GpioInterruptsClass::attachInterrupt(ULONG pin, std::function<void(void)> func, ULONG mode, HANDLE hController)
{
	HRESULT hr = S_OK;

	DMAP_ATTACH_INTERRUPT_BUFFER data;
	data.IntNo = (uint16_t)pin;
	data.Mode = (uint16_t)mode;

	DWORD info = 0;
	BOOL ss = DeviceIoControl(hController, IOCTL_DMAP_ATTACH_INTERRUPT,
		&data, sizeof(DMAP_ATTACH_INTERRUPT_BUFFER), nullptr, 0, &info, nullptr);
	if (ss == TRUE) {
		_future = std::async(std::launch::async, [pin, func, mode, hController](){
			HRESULT hr = S_OK;
			DMAP_WAIT_INTERRUPT_REQUEST_BUFFER input; input.IntNo = pin;
			DMAP_WAIT_INTERRUPT_NOTIFY_BUFFER output;
			
			DWORD info = 0;

			// Until the driver starts cancelling interrupt wait requests on this pin:
			BOOL ioc;
			do{
				ioc = DeviceIoControl(hController, IOCTL_DMAP_WAIT_INTERRUPT,
					&input, sizeof(DMAP_WAIT_INTERRUPT_REQUEST_BUFFER), 
					&output, sizeof(DMAP_WAIT_INTERRUPT_NOTIFY_BUFFER), &info, nullptr);
				if (ioc == TRUE) {
					func();
				}
			} while (ioc);
		});
	} else {
		hr = HRESULT_FROM_WIN32(GetLastError());
	}
	return hr;
}

/// Method to detach an interrupt for a GPIO port bit.
HRESULT GpioInterruptsClass::detachInterrupt(ULONG pin, HANDLE hController)
{
	DMAP_DETACH_INTERRUPT_BUFFER data;
	data.IntNo = (uint16_t)pin;

	DWORD info = 0;
	BOOL ss = DeviceIoControl(hController, IOCTL_DMAP_DETACH_INTERRUPT,
		&data, sizeof(DMAP_DETACH_INTERRUPT_BUFFER), nullptr, 0, &info, nullptr);

	_future.get();

	if (ss == TRUE) return S_OK;

	return HRESULT_FROM_WIN32(GetLastError());
}
