#include <Windows.h>
#include <future>
#include "lightning/GpioController.h"
#include "lightning/BcmSpiController.h"
#include "lightning/WindowsTime.h"
#include "HidInjector.h"

const wchar_t waitName[] = L"thit-wait";
const wchar_t mutexName[] = L"thit-mutex";

HidInjector hi;
HANDLE waitHandle = INVALID_HANDLE_VALUE;

BcmGpioControllerClass bgc;
BcmSpiControllerClass spi;

void SwitchAsync()
{
	bgc.setPinMode(6, INPUT);

	// Chattering measures
	ULONG s1, s2, ls = 10;
	while (waitHandle != INVALID_HANDLE_VALUE) {
		bgc.getPinState(6, s1);

		Sleep(10);

		bgc.getPinState(6, s2);

		if (s1 == s2) {
			if (ls != s1) {
				if (s1 == HIGH) {
					OutputDebugString(L"1");
#ifdef MOUSE_TEST
					hi.MouseDown(MOUSE_BUTTON_2);
#else
					hi.TouchDown(0);
#endif
				} else {
					OutputDebugString(L"0");
#ifdef MOUSE_TEST
					hi.MouseUp(MOUSE_BUTTON_2);
#else
					hi.TouchUp(0);
#endif
				}
			}
			ls = s1;
		}
	}
}

void InputAsync()
{
	spi.begin(3, 5000000);

	bgc.setPinMode(5, OUTPUT);

	ULONG send = 0b01'10'1'000'00000000;
	ULONG recv = 0x00;
	int dx = 0, dy = HidInjector::GetScreenHeight() / 2;

	while (waitHandle != INVALID_HANDLE_VALUE) {
		Sleep(10);

		bgc.setPinState(5, LOW);
		spi.transfer16(send, recv);
		bgc.setPinState(5, HIGH);

		dx = 100 + (HidInjector::GetScreenWidth() - 200) * recv / 1023;

		// If you want to get Y position.
		/*
		ULONG sendBufy = 0b01'11'1'000'00000000;
		bgc.setPinState(5, LOW);
		spi.transfer16(sendBuf, recvBuf);
		bgc.setPinState(5, HIGH);
		dy = 100 + (HidInjector::GetScreenHeight() - 200) * recv / 1023;
		*/

#ifdef MOUSE_TEST
		hi.MouseMove(dx, dy);
#else
		hi.TouchMove(0, dx, dy);
#endif
	}

	spi.end();
}

int main(int argc, char *argv[])
{
	HANDLE mutex = CreateMutex(NULL, FALSE, mutexName);
	if (GetLastError() == ERROR_ALREADY_EXISTS) {
		if (argc >= 2 && argv[1] == "e") {
			waitHandle = OpenEvent(EVENT_MODIFY_STATE, FALSE, waitName);
			if (waitHandle != NULL) SetEvent(waitHandle);
		}
		return 1;
	}

	if (hi.Open() == FALSE) return 1;

	waitHandle = CreateEvent(NULL, TRUE, FALSE, waitName);

	// Send "tnksoft" keystroke
	UsbKeys uks[] = { UsbKeys::T, UsbKeys::N, UsbKeys::K, UsbKeys::S, UsbKeys::O, UsbKeys::F, UsbKeys::T };
	for (int i = 0; i < 7; i++) {
		hi.KeyDown(0, uks[i]);
		hi.KeyUp(0);
	}

	auto a1 = std::async(std::launch::async, InputAsync);
	auto a2 = std::async(std::launch::async, SwitchAsync);

	WaitForSingleObject(waitHandle, INFINITE);
	CloseHandle(waitHandle);
	waitHandle = INVALID_HANDLE_VALUE;

	a1.get();
	a2.get();

	hi.Close();

	return 0;
}